EXEC [__AddStoredProcInfo]
    /* StoredProc_Name         */ 'NuGetPackages_CreateOrUpdatePackage',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'PackageHashRequired_Indicator',
    /* Description_Text        */ 'Creates a new NuGet package or updates an existing one.'
GO

IF OBJECT_ID('[NuGetPackages_CreateOrUpdatePackage]') IS NOT NULL DROP PROCEDURE [NuGetPackages_CreateOrUpdatePackage]
GO

SET QUOTED_IDENTIFIER ON
GO
SET ANSI_NULLS ON
GO

CREATE PROCEDURE [NuGetPackages_CreateOrUpdatePackage]
(
	@Feed_Id INT,
	@Package_Id NVARCHAR(255),
	@Version_Text NVARCHAR(50),
	@TargetFrameworks_Text VARCHAR(255),
	@Published_Date DATETIME,
	@Package_Size BIGINT,
	@PackageHash_Bytes BINARY(64),
	@NuspecFile_Bytes VARBINARY(MAX),
	@Symbols_Indicator YNINDICATOR,
	@Source_Indicator YNINDICATOR,
	@Cached_Indicator YNINDICATOR,
	@SymbolFiles_Xml XML,
	@Listed_Indicator YNINDICATOR = NULL,
	@Download_Count INT = NULL,
	@PackageHash_SHA1_Bytes BINARY(20) = NULL,
	@PackageHashRequired_Indicator YNINDICATOR = NULL OUT
)
AS
BEGIN

	--<SymbolFiles>
	--  <SymbolFile Id="Guid" FileName="SymbolFileName" Path="SymbolFileNamePath" Age="Age" />
	--</SymbolFiles>

SET NOCOUNT ON
SET QUOTED_IDENTIFIER ON

BEGIN TRY
	BEGIN TRANSACTION

	DECLARE @MaybeHashRequired YNINDICATOR

	SET @PackageHashRequired_Indicator = 'N'

	SET @Version_Text = (SELECT CAST((CAST([Major] AS NVARCHAR(10))
	                         + (CASE WHEN [Minor] IS NOT NULL THEN '.' + CAST([Minor] AS NVARCHAR(10)) ELSE '' END)
						     + (CASE WHEN [Patch] IS NOT NULL THEN '.' + CAST([Patch] AS NVARCHAR(10)) ELSE '' END)
						     + (CASE WHEN [Build] IS NOT NULL THEN '.' + CAST([Build] AS NVARCHAR(10)) ELSE '' END)
						     + (CASE WHEN [Prerelease] IS NOT NULL THEN '-' + [Prerelease] ELSE '' END)) AS NVARCHAR(50))
	                       FROM dbo.ParseNuGetVersion(@Version_Text))

	SET @MaybeHashRequired = (SELECT CASE WHEN [PackageHash_SHA1_Bytes] IS NULL THEN 'Y' ELSE 'N' END
								FROM [NuGetPackages]
							    WITH (UPDLOCK)
							   WHERE [Feed_Id] = @Feed_Id
								 AND [Package_Id] = @Package_Id
								 AND [Version_Text] = @Version_Text)

	IF @MaybeHashRequired IS NOT NULL
	BEGIN

		SET @PackageHashRequired_Indicator = @MaybeHashRequired

		UPDATE [NuGetPackages]
		   SET [TargetFrameworks_Text] = NULLIF(@TargetFrameworks_Text, ''),
			   [Published_Date] = COALESCE(@Published_Date, [Published_Date]),
			   [Package_Size] = @Package_Size,
			   [PackageHash_Bytes] = COALESCE(@PackageHash_Bytes, [PackageHash_Bytes]),
			   [PackageHash_SHA1_Bytes] = COALESCE(@PackageHash_SHA1_Bytes, [PackageHash_SHA1_Bytes]),
		       [NuspecFile_Bytes] = @NuspecFile_Bytes,
		       [Symbols_Indicator] = @Symbols_Indicator,
		       [Source_Indicator] = @Source_Indicator,
			   [Cached_Indicator] = COALESCE(@Cached_Indicator, [Cached_Indicator]),
			   [Listed_Indicator] = COALESCE(@Listed_Indicator, [Listed_Indicator])
		 WHERE [Feed_Id] = @Feed_Id
		   AND [Package_Id] = @Package_Id 
		   AND [Version_Text] = @Version_Text

		DELETE [NuGetPackageSymbols]
		 WHERE [Feed_Id] = @Feed_Id
		   AND [Package_Id] = @Package_Id
		   AND [Package_Version_Text] = @Version_Text


		IF COALESCE(dbo.Session_GetUserName(), 'UNKNOWN') NOT IN ('SYSTEM', 'UNKNOWN')
			EXEC [Events_RaiseEvent] 'PKGMDF', @Feed_Id, 'Package_Name', @Package_Id, 'Package_Version', @Version_Text

	END

	ELSE
	BEGIN

		IF @PackageHash_Bytes IS NULL OR @PackageHash_SHA1_Bytes IS NULL
		BEGIN
			SET @PackageHashRequired_Indicator = 'Y'
		END
		ELSE
		BEGIN
			INSERT INTO [NuGetPackages] 
			(
				[Feed_Id],
				[Package_Id],
				[TargetFrameworks_Text],
				[Published_Date],
				[Package_Size],
				[Download_Count],
				[NuspecFile_Bytes],
				[PackageHash_Bytes],
				[PackageHash_SHA1_Bytes],
				[Symbols_Indicator],
				[Source_Indicator],
				[Major_Number],
				[Minor_Number],
				[Patch_Number],
				[Build_Number],
				[Prerelease_Text],
				[Cached_Indicator],
				[Listed_Indicator]
			)
			SELECT [Feed_Id] = @Feed_Id,
			       [Package_Id] = @Package_Id,
				   [TargetFrameworks_Text] = NULLIF(@TargetFrameworks_Text, ''),
				   [Published_Date] = COALESCE(@Published_Date, GETUTCDATE()),
				   [Package_Size] = @Package_Size,
				   [Download_Count] = COALESCE(@Download_Count, 0),
				   [NuspecFile_Bytes] = @NuspecFile_Bytes,
				   [PackageHash_Bytes] = @PackageHash_Bytes,
				   [PackageHash_SHA1_Bytes] = @PackageHash_SHA1_Bytes,
				   [Symbols_Indicator] = @Symbols_Indicator,
				   [Source_Indicator] = @Source_Indicator,
				   [Major_Number] = V.[Major],
				   [Minor_Number] = V.[Minor],
				   [Patch_Number] = V.[Patch],
				   [Build_Number] = V.[Build],
				   [Prerelease_Text] = V.[Prerelease],
				   [Cached_Indicator] = COALESCE(@Cached_Indicator, 'N'),
				   [Listed_Indicator] = 'Y'
			  FROM dbo.ParseNuGetVersion(@Version_Text) V
		END

		EXEC [Events_RaiseEvent] 'PKGADD', @Feed_Id, 'Package_Name', @Package_Id, 'Package_Version', @Version_Text

	END

	IF @PackageHashRequired_Indicator <> 'Y'
	BEGIN
		INSERT INTO [NuGetPackageSymbols]
		(
			[Feed_Id],
			[Symbol_Id],
			[SymbolFileName_Text],
			[Package_SymbolFilePath_Text],
			[Symbol_Age],
			[Package_Id],
			[Package_Version_Text]
		)
		SELECT @Feed_Id,
		       SFX.[SymbolFile].[value]('@Id', 'BINARY(16)'),
		       SFX.[SymbolFile].[value]('@FileName', 'NVARCHAR(255)'),
		       SFX.[SymbolFile].[value]('@Path', 'NVARCHAR(255)'),
		       SFX.[SymbolFile].[value]('@Age', 'INT'),
		       @Package_Id,
		       @Version_Text
		  FROM @SymbolFiles_Xml.[nodes]('/SymbolFiles/SymbolFile') AS SFX([SymbolFile])
		
	END

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH


END
GO

GRANT EXECUTE ON [NuGetPackages_CreateOrUpdatePackage] TO [ProGetUser_Role]